#include <PalmOS.h>
#include "res.h"
#include "persist.h"
#include "prefsmodal.h"

static void MainFormInit(FormType *form){
    struct myPrefs prefs;
    // ensure default prefs are set
    MyPrefsGet(&prefs);
    MyPrefsSet(&prefs);
}

static void MainFormDeInit(FormType *form){
}

#pragma pack(push,1)
struct books_row {
    UInt8 period;
    UInt8 book1, book2, book3;
};
#pragma pack(pop)

static void MainFormUpdateWithData(FormType *form, UInt16 day, UInt16 startDay){
    MemHandle booksH = DmGetResource('tSTR', BookData);
    struct books_row *books;
    RectangleType bounds;
    DateType date;
    UInt16 idx;
    ControlType *control;
    char *buf;

#define _GETLABEL(controlID) \
    idx = FrmGetObjectIndex(form, controlID); \
    control = FrmGetObjectPtr(form, idx); \
    buf = (char *)CtlGetLabel(control);

    _GETLABEL(DayValue)
    FrmHideObject(form, idx);
    StrPrintF(buf, "%d", 1+day);
    CtlSetLabel(control, buf);
    FrmShowObject(form, idx);

    _GETLABEL(DateValue)
    FrmHideObject(form, idx);
    date.year = 1;
    date.month = 1;
    date.day = 1;
    DateAdjust(&date, (startDay + day) % 365);
    DateToAscii(date.month, date.day, date.year, dfMDYLongWithComma, buf);
    for(char *c = buf + 5; c < buf + 7; ++c){
        if(*c == ','){
            *c = 0;
            break;
        }
    }
    CtlSetLabel(control, buf);
    FrmShowObject(form, idx);

    books = MemHandleLock(booksH);
    _GETLABEL(PeriodLabel)
    FrmHideObject(form, idx);
    SysStringByIndex(PeriodStrings, books[day].period, buf, 22);
    CtlSetLabel(control, buf);
    FrmShowObject(form, idx);

    // hide chapters first so we don't erase books
    _GETLABEL(Chapter1)
    FrmHideObject(form, idx);
    _GETLABEL(Chapter2)
    FrmHideObject(form, idx);
    _GETLABEL(Chapter3)
    FrmHideObject(form, idx);

    _GETLABEL(Book1)
    FrmHideObject(form, idx);
    SysStringByIndex(BookStrings, books[day].book1, buf, 17);
    CtlSetLabel(control, buf);
    FrmShowObject(form, idx);
    FrmGetObjectBounds(form, idx, &bounds);
    _GETLABEL(Chapter1)
    FrmSetObjectPosition(form, idx, bounds.topLeft.x + bounds.extent.x, bounds.topLeft.y);
    SysStringByIndex(Chapter1Data, day, buf, 12);
    CtlSetLabel(control, buf);
    FrmShowObject(form, idx);
    _GETLABEL(Book2)
    FrmHideObject(form, idx);
    SysStringByIndex(BookStrings, books[day].book2, buf, 17);
    CtlSetLabel(control, buf);
    FrmShowObject(form, idx);
    FrmGetObjectBounds(form, idx, &bounds);
    _GETLABEL(Chapter2)
    FrmSetObjectPosition(form, idx, bounds.topLeft.x + bounds.extent.x, bounds.topLeft.y);
    SysStringByIndex(Chapter2Data, day, buf, 12);
    CtlSetLabel(control, buf);
    FrmShowObject(form, idx);
    _GETLABEL(Book3)
    FrmHideObject(form, idx);
    SysStringByIndex(BookStrings, books[day].book3, buf, 17);
    CtlSetLabel(control, buf);
    FrmShowObject(form, idx);
    FrmGetObjectBounds(form, idx, &bounds);
    _GETLABEL(Chapter3)
    FrmSetObjectPosition(form, idx, bounds.topLeft.x + bounds.extent.x, bounds.topLeft.y);
    SysStringByIndex(Chapter3Data, day, buf, 12);
    CtlSetLabel(control, buf);
    FrmShowObject(form, idx);

    MemHandleUnlock(booksH);

#undef _GETLABEL
}
static void MainFormUpdate(FormType *form){
    struct myPrefs prefs;
    MyPrefsGet(&prefs);
    MainFormUpdateWithData(form, prefs.day, prefs.startDay);
}

static void _Increment(FormType *form, Int8 d){
    struct myPrefs prefs;
    MyPrefsGet(&prefs);
    prefs.day = (prefs.day + 365 + d) % 365;
    MainFormUpdateWithData(form, prefs.day, prefs.startDay);
    MyPrefsSet(&prefs);
}

Boolean MainFormHandleEvent(EventType *event){
    FormType *form, *dialog;
    UInt16 choice;
    switch(event->eType){
    case frmOpenEvent:
        form = FrmGetActiveForm();
        MainFormInit(form);
        MainFormUpdate(form);
        FrmDrawForm(form);
        return true;
    case frmUpdateEvent:
        form = FrmGetActiveForm();
        FrmDrawForm(form);
        return true;
    case frmCloseEvent:
        form = FrmGetActiveForm();
        MainFormDeInit(form);
        return false;
    case ctlSelectEvent:
        switch(event->data.ctlSelect.controlID){
        case PrefsBtn:
            form = FrmGetActiveForm();
            dialog = FrmInitForm(PrefsModal);
            PrefsModalInit(dialog);
            FrmSetEventHandler(dialog, PrefsModalHandleEvent);
            choice = FrmDoDialog(dialog);
            if(choice == OKBtn){
                PrefsModalSave(dialog);
                MainFormUpdate(form);
            }
            FrmDeleteForm(dialog);
            return true;
        case AboutBtn:
            dialog = FrmInitForm(AboutDialog);
            FrmDoDialog(dialog);
            FrmDeleteForm(dialog);
            return true;
        }
        return false;
    case ctlRepeatEvent:
        form = FrmGetActiveForm();
        switch(event->data.ctlRepeat.controlID){
        case UpArrow:
            _Increment(form, 1);
            break;
        case DownArrow:
            _Increment(form, -1);
            break;
        }
        return false;
    case keyDownEvent:
        form = FrmGetActiveForm();
        switch(event->data.keyDown.chr){
        case pageUpChr:
            _Increment(form, 1);
            break;
        case pageDownChr:
            _Increment(form, -1);
            break;
        default: break;
        }
        return true;
    default:
        return false;
    }
}
